/*
 * Copyright 2015-2016 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import ru.ispras.fortress.data.types.bitvector.BitVector;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.randomizer.Variate;

import java.math.BigInteger;

final class GeneratorRandomBiased extends DataGeneratorBase {
  public static final String NAME = "random_biased";

  public GeneratorRandomBiased() {
    super(NAME);
  }

  @Override
  protected void generate(final DataGeneratorQuery query) {
    final Variate<?> dist = (Variate<?>) query.getParameter("dist");

    final int size = query.hasParameter("size")
        ? query.getIntegerParameter("size").intValue() : -1;

    for (final DataGeneratorVariable variable : query.getUnknownVariables()) {
      final int valueSize = -1 != size ? size : variable.getSize();
      final BitVector value = toBitVector(dist.value(), valueSize);
      setValue(variable, NodeValue.newBitVector(value));
    }
  }

  private static BitVector toBitVector(final Object value, final int size) {
    if (value instanceof Integer) {
      return BitVector.valueOf((Integer) value, size);
    }

    if (value instanceof Long) {
      return BitVector.valueOf((Long) value, size);
    }

    if (value instanceof BigInteger) {
      return BitVector.valueOf((BigInteger) value, size);
    }

    throw new IllegalArgumentException(
        "Unsupported value type: " + value.getClass().getName());
  }
}
